﻿using Syncfusion.DocIO.DLS;
using Syncfusion.UI.Xaml.RichTextBoxAdv;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace Sample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            // Loads the document from stream.
            Assembly assembly = typeof(MainPage).GetTypeInfo().Assembly;
            //Adding the language dictionary to spellchecker.
            richTextBoxAdv.SpellChecker.Dictionaries.Add("ms-appx:///Assets/en_US.dic");
            richTextBoxAdv.SpellChecker.IsEnabled = false;
            //Stream stream = assembly.GetManifestResourceStream("Sample.Assets.GettingStarted.docx");
            //richTextBoxAdv.Load(stream, Syncfusion.UI.Xaml.RichTextBoxAdv.FormatType.Docx);
        }
        /// <summary>
        /// Called when the button is clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void open_Click(object sender, RoutedEventArgs e)
        {
            FileOpenPicker fileOpenPicker = new FileOpenPicker();
            fileOpenPicker.FileTypeFilter.Add(".doc");
            fileOpenPicker.FileTypeFilter.Add(".dot");
            fileOpenPicker.FileTypeFilter.Add(".docx");
            fileOpenPicker.FileTypeFilter.Add(".dotx");
            fileOpenPicker.FileTypeFilter.Add(".rtf");
            fileOpenPicker.FileTypeFilter.Add(".htm");
            fileOpenPicker.FileTypeFilter.Add(".html");
            fileOpenPicker.FileTypeFilter.Add(".xml");
            fileOpenPicker.FileTypeFilter.Add(".txt");
            StorageFile stgFile = await fileOpenPicker.PickSingleFileAsync();
            if (stgFile != null)
            {
                WordDocument wordDocument = new WordDocument();
                if(stgFile.FileType == ".docx" || stgFile.FileType == ".doc")
                {
                    Stream openStream = await stgFile.OpenStreamForReadAsync();

                    if (stgFile.FileType == ".doc")
                        wordDocument = new WordDocument(openStream, Syncfusion.DocIO.FormatType.Doc);
                    else
                        wordDocument = new WordDocument(openStream, Syncfusion.DocIO.FormatType.Docx);

                    //Gets the owner paragrph of PageBreak.
                    WParagraph paragraph = GetPageBreakParagraph(wordDocument.LastSection.Body);

                    //Insert section break and remove the page break from owner paragraph.
                    if (paragraph != null)
                    {
                        paragraph.InsertSectionBreak(SectionBreakCode.NewPage);
                        RemovePageBreak(paragraph);
                    }
                    //Creates new memory stream.
                    Stream ms = new MemoryStream();

                    //Save the document as Docx stream.
                    wordDocument.Save(ms, Syncfusion.DocIO.FormatType.Docx);

                    //Reset teh stream position.
                    ms.Position = 0;
                    //Close the DocIO instance.
                    wordDocument.Close();

                    try
                    {
                        //Open the Docx stream in SfRichTextBoxAdv .
                        Task<bool> loadAsync = this.richTextBoxAdv.LoadAsync(ms, FormatType.Docx);
                        await loadAsync;
                    }
                    catch (Exception)
                    {
                    }
                }
                else
                {
                    try
                    {
                        Task<bool> loadAsync = this.richTextBoxAdv.LoadAsync(stgFile);
                        await loadAsync;
                    }
                    catch (Exception)
                    {
                    }
                }
            }
        }
        /// <summary>
        /// Remove the page break from the paragraph.
        /// </summary>
        /// <param name="paragraph"></param>
        private static void RemovePageBreak(WParagraph paragraph)
        {
            for (int index = 0; index < paragraph.ChildEntities.Count; index++)
            {
                if (paragraph.ChildEntities[index] is Break && (paragraph.ChildEntities[index] as Break).BreakType == BreakType.PageBreak)
                {
                    paragraph.ChildEntities.RemoveAt(index);
                    return;
                }
            }
        }
        /// <summary>
        /// Find the paragraph which has page break.
        /// </summary>
        /// <param name="body"></param>
        /// <returns></returns>
        private static WParagraph GetPageBreakParagraph(WTextBody body)
        {
            foreach (WParagraph paragraph in body.Paragraphs)
            {
                foreach (Entity paragraphItem in paragraph.ChildEntities)
                {
                    if (paragraphItem is Break && (paragraphItem as Break).BreakType == BreakType.PageBreak)
                        return paragraph;
                }
            }
            return null;
        }
    }
}
